%
%
% [valgamma valX] =
% brlOpt1step(Ag,Bg,Cg,Dg,Ap,Bp,Cp,Dp,Ao,Bo,Co,Do,Ak,Bk,Ck,Dk)
%
%
% Inpus: state space representation of the system G: Ag,Bg,Cg,Dg
%        state space representation of the uncertainty wo and performance
%          weight wp: Ao,Bo,Co,Do,Ap,Bp,Cp,Dp
%        state space representation of the controller K: Ak,Bk,Ck,Dk
%
%
% Initialize the LMI
% | A'*X+X*A      X*B    C'*gamma+C0|
% |  B'*X         -I     D'*gamma+D0| <1
% |gamma*C+C0 gamma*D+D0     -I     |
% gamma>0
%     X>0
%
% This LMI is builded form the input variables. X and gamma is the LMI
% variables. A,B,C,D is the state space representation of the closed loop
% system 
%
% Solving the minimization problem
%
%                 min c'x
%                   s.t.
%    
%   | A'*X+X*A      X*B    C'*gamma+C0|
%   |  B'*X         -I     D'*gamma+D0| <1
%   |gamma*C+C0 gamma*D+D0     -I     |
% 
%               gamma>0
%                   X>0
%
% using MINCX, where gamma and the elements in X is putted into the vector x.
%
% Returning: - valgamma: the value of gamma that gives the optimal
%              objective value
%            - valX: The value of X that gives the optimal objective value 

function [valgamma valX] = brlOpt1step(Ag,Bg,Cg,Dg,Ap,Bp,Cp,Dp,Ao,Bo,Co,Do,Ak,Bk,Ck,Dk)
%% System matrix decleration
Agsize = size(Ag);
Apsize = size(Ap);
Aosize = size(Ao);
WpbGcsize = size(-Bp*Cg);
WobGcsize = size(Bo*Cg);
Wpbsize = size(Bp);
Wpcsize = size(Cp);
Wocsize = size(Co);
WodGcsize = size(Do*Cg);

A = [Ag zeros(Agsize(1),Apsize(2)) zeros(Agsize(1),Aosize(2));
     -Bp*Cg Ap zeros(WpbGcsize(1),Aosize(2));
     Bo*Cg zeros(WobGcsize(1),Apsize(2)) Ao];

B1 = [zeros(Agsize(1),Wpbsize(2));Bp;zeros(Aosize(1),Wpbsize(2))];
B2 = [Bg;-Bp*Dg;Bo*Dg];

C1u = [zeros(WodGcsize(1),WodGcsize(2)) zeros(Wpcsize(1),Wpcsize(2)) zeros(Wocsize(1),Wocsize(2));
       Do*Cg zeros(Wpcsize(1),Wpcsize(2)) Co];

C2 = [-Cg zeros(2,length(A)-length(Cg))];

gammaC1 = [-Dp*Cg Cp zeros(Wocsize(1),Wocsize(2));
           zeros(WodGcsize(1),WodGcsize(2)) zeros(Wpcsize(1),Wpcsize(2)) zeros(Wocsize(1),Wocsize(2))];

gammaD11 = [Dp; zeros(WodGcsize(1),length(Dp))];

Acl = [A+B2*Dk*C2 B2*Ck;
       Bk*C2 Ak];

Bcl = [B1+B2*Dk;
       Bk];

C1usize=size(C1u);
Cclu = [C1u zeros(C1usize(1),length(Ak))];
Cclgamma = [gammaC1 zeros(C1usize(1),length(Ak))];
Dclgamma = gammaD11;

% Bare for test %
Dcl = Dclgamma;
Ccl = Cclu+Cclgamma;

[Dclrow Dclcol] = size(Dclgamma);
%% Declaring and initialize LMI variables
setlmis([]);
[gamma,kn,sgamma] = lmivar(1,[1 0]);
[X,kn,sX] = lmivar(1,[length(Acl) 1]);

%% LMIs in eq(29)
% 1st LMI 
BRL = newlmi; %LMI form the Bounded Real Lemma
lmiterm([BRL 1 1 X],1,Acl,'s') % Acl'X+XAcl
lmiterm([BRL 2 1 X],Bcl',1);
lmiterm([BRL 2 2 0],-1);
lmiterm([BRL 3 1 gamma],1,Cclgamma);
lmiterm([BRL 3 1 0],Cclu);
lmiterm([BRL 2 3 gamma],1,Dclgamma');
lmiterm([BRL 3 3 0],-1);

% 2nd LMI 
Xpos = newlmi;
lmiterm([-Xpos 1 1 X],1,1);

% 3nd LMI 
gammapos = newlmi;
lmiterm([-gammapos 1 1 gamma],1,1);

lmisys = getlmis;

%% Getting C vector
ndec = decnbr(lmisys);
c = zeros(ndec,1);

gamma1 = 0;
i =1; 
while gamma1==0
  [gamma1 V] = defcx(lmisys,i,gamma,X);  
end
c(i) = -1;
clear i;

%% Finding minimum value
% M hente ut en feasable lsning som startpunkt for mincx. Hvis ikke, blir
% mincx infeaseble.

[copt,xopt] = feasp(lmisys);
[copt,xopt] = mincx(lmisys,c,[],xopt);

%% Variables to be returned
valX = dec2mat(lmisys,xopt,X);
valgamma = dec2mat(lmisys,xopt,gamma);

    