%%  Testing feasibility for the solutions Ak,Bk,Ck,Dk,X0
%
% [tmin xfeas] =
% brlfeas(Ag,Bg,Cg,Dg,Ap,Bp,Cp,Dp,Ao,Bo,Co,Do,Ak,Bk,Ck,Dk,X0)
%
% Inpus: - state space representation of the system G: Ag,Bg,Cg,Dg
%        - state space representation of the uncertainty wo and performance
%          weight wp: Ao,Bo,Co,Do,Ap,Bp,Cp,Dp
%        - state space representation of the controller K: Ak,Bk,Ck,Dk
%        - the positive definite symetric matrix X0
%
% Testing feasibility of the LMI
% | A'*X0+X0*A    X0*B    C'*gamma+C0|
% |  B'*X0         -I     D'*gamma+D0| <1
% |gamma*C+C0 gamma*D+D0      -I     |
% 
% gamma > 0
% This LMI is builded form the input variables, and gamma is the only LMI
% variable. A,B,C,D is the state space representation of the closed loop system 
%
% Returning: - tmin>=0 if infeasible (Marginal feasible or infeasible)
%              tmin<0 if feasible
%            - xfeas is a variable that makes the LMI feasible

function [tmin xfeas] = brlfeas(Ag,Bg,Cg,Dg,Ap,Bp,Cp,Dp,Ao,Bo,Co,Do,Ak,Bk,Ck,Dk,X0)
%% System matrix decleration
Agsize = size(Ag);
Apsize = size(Ap);
Aosize = size(Ao);
WpbGcsize = size(-Bp*Cg);
WobGcsize = size(Bo*Cg);
Wpbsize = size(Bp);
Wpcsize = size(Cp);
Wocsize = size(Co);
WodGcsize = size(Do*Cg);

A = [Ag zeros(Agsize(1),Apsize(2)) zeros(Agsize(1),Aosize(2));
     -Bp*Cg Ap zeros(WpbGcsize(1),Aosize(2));
     Bo*Cg zeros(WobGcsize(1),Apsize(2)) Ao];

B1 = [zeros(Agsize(1),Wpbsize(2));Bp;zeros(Aosize(1),Wpbsize(2))];
B2 = [Bg;-Bp*Dg;Bo*Dg];

C1u = [zeros(WodGcsize(1),WodGcsize(2)) zeros(Wpcsize(1),Wpcsize(2)) zeros(Wocsize(1),Wocsize(2));
       Do*Cg zeros(Wpcsize(1),Wpcsize(2)) Co];

C2 = [-Cg zeros(2,length(A)-length(Cg))];

gammaC1 = [-Dp*Cg Cp zeros(Wocsize(1),Wocsize(2));
           zeros(WodGcsize(1),WodGcsize(2)) zeros(Wpcsize(1),Wpcsize(2)) zeros(Wocsize(1),Wocsize(2))];

gammaD11 = [Dp; zeros(WodGcsize(1),length(Dp))];

Acl = [A+B2*Dk*C2 B2*Ck;
       Bk*C2 Ak];

Bcl = [B1+B2*Dk;
       Bk];

C1usize=size(C1u);
Cclu = [C1u zeros(C1usize(1),length(Ak))];
Cclgamma = [gammaC1 zeros(C1usize(1),length(Ak))];
Dclgamma = gammaD11;

% Bare for test %
Dcl = Dclgamma;
Ccl = Cclu+Cclgamma;

[Dclrow Dclcol] = size(Dclgamma);

%% Initalizing gamma, the only LMI variable for this system
setlmis([]);
[gamma,kn,sgamma] = lmivar(1,[1 0]);
%% Building LMIs
% 1st LMI
BRL = newlmi;  % LMI form the Bounded Real Lemma
A1 = (Acl'*X0)+(X0*Acl);
lmiterm([BRL 1 1 0],A1)  % Acl'*X + X*Acl
B1 = X0*Bcl;
lmiterm([BRL 1 2 0],B1); % Bcl'*X
lmiterm([BRL 1 3 gamma],Cclgamma',1);
lmiterm([BRL 1 3 0],Cclu');
lmiterm([BRL 2 2 0],-1); % -eye(Dclcol)
lmiterm([BRL 2 3 gamma],Dclgamma',1);
lmiterm([BRL 3 3 0],-1); % -eye(Dclrow)

% 2nd LMI
% Xpos = newlmi;
% lmiterm([-Xpos 1 1 X],1,1);

% 3nd LMI
gammapos = newlmi;
lmiterm([-gammapos 1 1 gamma],1,1);

lmisys = getlmis;

%% Testing feasibility to the system with Ak Bk Ck Dk X0
disp('In brlfeas: Feasibility checks starts')
[tmin,xfeas] = feasp(lmisys);

if tmin >= 0
    disp('ERROR in brlfeas, Optimization step infeasible')
    return;
elseif tmin<0
    disp('In brlfeas: Optimization step feasible')
    return;
else
    disp('ERROR: The LMI is neither feasible or infeasible. Check code')
    return;
end   